'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var uuid = require('uuid');
var _ = _interopDefault(require('lodash'));
var react = require('react');

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function reducer(state, action) {
  var _extends2;

  switch (action.type) {
    case "insert":
      var id = action.item.id || uuid.v4();

      var data = _extends({}, state.data, (_extends2 = {}, _extends2[id] = {
        id: id,
        children: [],
        parentId: action.parentId,
        data: _extends({}, action.item, {
          id: id
        })
      }, _extends2));

      if (action.parentId) {
        var _data$action$parentId;

        data[action.parentId] = _extends({}, data[action.parentId], {
          children: [].concat((_data$action$parentId = data[action.parentId]) == null ? void 0 : _data$action$parentId.children, [id])
        });
      }

      return _extends({}, state, {
        root: _(data).values().filter(function (i) {
          return i && !i.parentId;
        }).map(function (i) {
          return i.id;
        }).valueOf(),
        data: data
      });

    case "push":
      var p = state.path;

      if (action.section !== undefined && p[action.section]) {
        p.splice(action.section, 1, action.item);
        p.splice(action.section + 1, p.length - action.section);
      } else {
        p.push(action.item);
      }

      return _extends({}, state, {
        path: [].concat(p)
      });

    case "pop":
      state.path.pop();
      return _extends({}, state, {
        path: [].concat(state.path)
      });

    default:
      return state;
  }
}

var getItem = function getItem(id, data) {
  var _data$id;

  return data == null ? void 0 : (_data$id = data[id]) == null ? void 0 : _data$id.data;
};

var getChildren = function getChildren(id, data, push) {
  var _data$id2;

  return createItemsProps((data == null ? void 0 : (_data$id2 = data[id]) == null ? void 0 : _data$id2.children) || [], data, push);
};

var createItemProps = function createItemProps(id, _data, push) {
  return {
    data: function data() {
      return getItem(id, _data);
    },
    children: function children() {
      return getChildren(id, _data, push);
    },
    pushAt: function pushAt(atSection) {
      return push(id, atSection);
    },
    buildProps: function buildProps(additional) {
      return _extends({}, additional, {
        key: id
      });
    }
  };
};
var createItemsProps = function createItemsProps(ids, data, push) {
  return {
    length: ids.length,
    includes: ids.includes,
    map: function map(callbackfn) {
      return ids.map(function (id, index) {
        return callbackfn(createItemProps(id, data, push), index, ids);
      });
    },
    forEach: function forEach(callbackfn) {
      return ids.forEach(function (id, index) {
        return callbackfn(createItemProps(id, data, push), index, ids);
      });
    }
  };
};
var buildOptions = function buildOptions(options) {
  if (!options) {
    return undefined;
  }

  var data = {}; // TODO
  // options.initialValues?.forEach((item) => {
  //     const id = uuid();
  //     data[item] = {
  //         id,
  //         children: [],
  //         parentId: action.parentId,
  //         data: _.omit(action.item, "id") as T
  //     };
  // });

  return {
    path: options.path || [],
    root: [],
    data: data
  };
};

/**
 * React Column View
 * Riccardo Caranfil
 *
 * Created by Riccardo Caranfil on 05/05/21
 */
var INITIAL_STATE = {
  path: [],
  root: [],
  data: {}
};
var useColumnView = function useColumnView(options) {
  var _useReducer = react.useReducer(reducer, buildOptions(options) || INITIAL_STATE),
      _useReducer$ = _useReducer[0],
      root = _useReducer$.root,
      data = _useReducer$.data,
      path = _useReducer$.path,
      dispatch = _useReducer[1];

  var insert = react.useCallback(function (item, parentId) {
    return dispatch({
      type: "insert",
      //@ts-ignore
      item: item,
      parentId: parentId
    });
  }, [dispatch]);
  var push = react.useCallback(function (item, section) {
    return dispatch({
      type: "push",
      item: item,
      section: section
    });
  }, [dispatch]);
  var pop = react.useCallback(function (item) {
    return dispatch({
      type: "pop",
      item: item
    });
  }, [dispatch]);
  return {
    // @ts-ignore
    root: createItemsProps(root, data, push),
    // @ts-ignore
    path: createItemsProps(path, data, push),
    insert: insert,
    push: push,
    pop: pop
  };
};

exports.StateReducer = reducer;
exports.useColumnView = useColumnView;
//# sourceMappingURL=react-column-view.cjs.development.js.map
